var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import atob from 'atob';
import { gql, useQuery, useSubscription, useApolloClient } from '@apollo/client/index.js';
import { generateApolloClient } from '@deep-foundation/hasura/client.js';
import { useLocalStore } from '@deep-foundation/store/local.js';
import React, { createContext, useContext, useMemo, useState } from "react";
import { deleteMutation, generateQuery, generateQueryData, generateSerial, insertMutation, updateMutation } from './gql/index.js';
import { MinilinkCollection, useMinilinksApply, useMinilinksQuery, useMinilinksSubscription } from './minilinks.js';
import { awaitPromise } from './promise.js';
import { useTokenController } from './react-token.js';
import { reserve } from './reserve.js';
import Debug from 'debug';
import { corePckg } from './core.js';
const debug = Debug('deeplinks:client');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = Debug.disable();
Debug.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
const corePckgIds = {};
corePckg.data.filter(l => !!l.type).forEach((l, i) => {
    corePckgIds[l.id] = i + 1;
});
export const _ids = {
    '@deep-foundation/core': corePckgIds,
};
export const _serialize = {
    links: {
        fields: {
            id: 'number',
            from_id: 'number',
            to_id: 'number',
            type_id: 'number',
        },
        relations: {
            from: 'links',
            to: 'links',
            type: 'links',
            in: 'links',
            out: 'links',
            typed: 'links',
            selected: 'selector',
            selectors: 'selector',
            value: 'value',
            string: 'value',
            number: 'value',
            object: 'value',
            can_rule: 'can',
            can_action: 'can',
            can_object: 'can',
            can_subject: 'can',
            down: 'tree',
            up: 'tree',
            tree: 'tree',
            root: 'tree',
        },
    },
    selector: {
        fields: {
            item_id: 'number',
            selector_id: 'number',
            query_id: 'number',
            selector_include_id: 'number',
        },
        relations: {
            item: 'links',
            selector: 'links',
            query: 'links',
        }
    },
    can: {
        fields: {
            rule_id: 'number',
            action_id: 'number',
            object_id: 'number',
            subject_id: 'number',
        },
        relations: {
            rule: 'links',
            action: 'links',
            object: 'links',
            subject: 'links',
        }
    },
    tree: {
        fields: {
            id: 'number',
            link_id: 'number',
            tree_id: 'number',
            root_id: 'number',
            parent_id: 'number',
            depth: 'number',
            position_id: 'string',
        },
        relations: {
            link: 'links',
            tree: 'links',
            root: 'links',
            parent: 'links',
            by_link: 'tree',
            by_tree: 'tree',
            by_root: 'tree',
            by_parent: 'tree',
            by_position: 'tree',
        }
    },
    value: {
        fields: {
            id: 'number',
            link_id: 'number',
            value: 'value',
        },
        relations: {
            link: 'links',
        },
    },
};
export const _boolExpFields = {
    _and: true,
    _not: true,
    _or: true,
};
export const pathToWhere = (start, ...path) => {
    var _a, _b;
    const pckg = typeof (start) === 'string' ? { type_id: (_a = _ids === null || _ids === void 0 ? void 0 : _ids['@deep-foundation/core']) === null || _a === void 0 ? void 0 : _a.Package, value: start } : { id: start };
    let where = pckg;
    for (let p = 0; p < path.length; p++) {
        const item = path[p];
        if (typeof (item) !== 'boolean') {
            const nextWhere = { in: { type_id: (_b = _ids === null || _ids === void 0 ? void 0 : _ids['@deep-foundation/core']) === null || _b === void 0 ? void 0 : _b.Contain, value: item, from: where } };
            where = nextWhere;
        }
    }
    return where;
};
export const serializeWhere = (exp, env = 'links') => {
    var _a, _b, _c, _d;
    if (Object.prototype.toString.call(exp) === '[object Array]')
        return exp.map((e) => serializeWhere(e, env));
    else if (typeof (exp) === 'object') {
        const keys = Object.keys(exp);
        const result = {};
        for (let k = 0; k < keys.length; k++) {
            const key = keys[k];
            const type = typeof (exp[key]);
            if (typeof (exp[key]) === 'undefined')
                throw new Error(`${key} === undefined`);
            let setted = false;
            const is_id_field = !!~['type_id', 'from_id', 'to_id'].indexOf(key);
            if (env === 'links') {
                if (type === 'string' || type === 'number') {
                    if (key === 'value' || key === type) {
                        setted = result[type] = { value: { _eq: exp[key] } };
                    }
                    else {
                        setted = result[key] = { _eq: exp[key] };
                    }
                }
                else if (!_boolExpFields[key] && Object.prototype.toString.call(exp[key]) === '[object Array]') {
                    setted = result[key] = serializeWhere(pathToWhere(...exp[key]));
                }
            }
            else if (env === 'tree') {
                if (type === 'string' || type === 'number') {
                    setted = result[key] = { _eq: exp[key] };
                }
                else if (!_boolExpFields[key] && Object.prototype.toString.call(exp[key]) === '[object Array]') {
                    setted = result[key] = serializeWhere(pathToWhere(...exp[key]));
                }
            }
            else if (env === 'value') {
                if (type === 'string' || type === 'number') {
                    setted = result[key] = { _eq: exp[key] };
                }
            }
            if (type === 'object' && exp[key].hasOwnProperty('_type_of') && ((env === 'links' && (is_id_field || key === 'id')) ||
                (env === 'selector' && key === 'item_id') ||
                (env === 'can' && !!~['rule_id', 'action_id', 'subject_id', 'object_id',].indexOf(key)) ||
                (env === 'tree' && !!~['link_id', 'tree_id', 'root_id', 'parent_id'].indexOf(key)) ||
                (env === 'value' && key === 'link_id'))) {
                const _temp = setted = { _by_item: { path_item_id: { _eq: exp[key]._type_of }, group_id: { _eq: 0 } } };
                if (key === 'id') {
                    result._and = result._and ? [...result._and, _temp] : [_temp];
                }
                else {
                    result[key.slice(0, -3)] = _temp;
                }
            }
            else if (type === 'object' && exp[key].hasOwnProperty('_id') && ((env === 'links' && (is_id_field || key === 'id')) ||
                (env === 'selector' && key === 'item_id') ||
                (env === 'can' && !!~['rule_id', 'action_id', 'subject_id', 'object_id',].indexOf(key)) ||
                (env === 'tree' && !!~['link_id', 'tree_id', 'root_id', 'parent_id'].indexOf(key)) ||
                (env === 'value' && key === 'link_id')) && Object.prototype.toString.call(exp[key]._id) === '[object Array]' && exp[key]._id.length >= 1) {
                const _temp = setted = serializeWhere(pathToWhere(exp[key]._id[0], ...exp[key]._id.slice(1)), 'links');
                if (key === 'id') {
                    result._and = result._and ? [...result._and, _temp] : [_temp];
                }
                else {
                    result[key.slice(0, -3)] = _temp;
                }
            }
            if (!setted) {
                const _temp = (_boolExpFields[key]) ? (serializeWhere(exp[key], env)) : ((_b = (_a = _serialize === null || _serialize === void 0 ? void 0 : _serialize[env]) === null || _a === void 0 ? void 0 : _a.relations) === null || _b === void 0 ? void 0 : _b[key]) ? (serializeWhere(exp[key], (_d = (_c = _serialize === null || _serialize === void 0 ? void 0 : _serialize[env]) === null || _c === void 0 ? void 0 : _c.relations) === null || _d === void 0 ? void 0 : _d[key])) : (exp[key]);
                if (key === '_and')
                    result._and ? result._and.push(..._temp) : result._and = _temp;
                else
                    result[key] = _temp;
            }
        }
        return result;
    }
    else {
        if (typeof (exp) === 'undefined')
            throw new Error('undefined in query');
        return exp;
    }
};
export const serializeQuery = (exp, env = 'links') => {
    const { limit, order_by, offset, distinct_on } = exp, where = __rest(exp, ["limit", "order_by", "offset", "distinct_on"]);
    const result = { where: typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : serializeWhere(where, env) : { id: { _eq: exp } } };
    if (limit)
        result.limit = limit;
    if (order_by)
        result.order_by = order_by;
    if (offset)
        result.offset = offset;
    if (distinct_on)
        result.distinct_on = distinct_on;
    return result;
};
export function parseJwt(token) {
    var base64Url = token.split('.')[1];
    var base64 = base64Url.replace(/-/g, '+').replace(/_/g, '/');
    var jsonPayload = decodeURIComponent(atob(base64).split('').map(function (c) {
        return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
    }).join(''));
    const parsed = JSON.parse(jsonPayload);
    const _a = parsed['https://hasura.io/jwt/claims'] || {}, { 'x-hasura-allowed-roles': roles, 'x-hasura-default-role': role, 'x-hasura-user-id': userId } = _a, other = __rest(_a, ['x-hasura-allowed-roles', 'x-hasura-default-role', 'x-hasura-user-id']);
    return Object.assign({ userId: +userId, role, roles }, other);
}
;
export function checkAndFillShorts(obj) {
    var _a, _b, _c;
    for (var i in obj) {
        if (!obj.hasOwnProperty(i))
            continue;
        if ((typeof obj[i]) == 'object' && obj[i] !== null) {
            if (typeof obj[i] === 'object' && i === 'object' && ((_b = (_a = obj[i]) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.value) === undefined) {
                obj[i] = { data: { value: obj[i] } };
                continue;
            }
            if (typeof obj[i] === 'object' && (i === 'to' || i === 'from' || i === 'in' || i === 'out') && ((_c = obj[i]) === null || _c === void 0 ? void 0 : _c.data) === undefined)
                obj[i] = { data: obj[i] };
            checkAndFillShorts(obj[i]);
        }
        else if (i === 'string' && typeof obj[i] === 'string' || i === 'number' && typeof obj[i] === 'number')
            obj[i] = { data: { value: obj[i] } };
    }
}
export class DeepClient {
    _silent(options = {}) {
        return typeof (options.silent) === 'boolean' ? options.silent : this.silent;
    }
    constructor(options) {
        var _a, _b, _c;
        this.useDeepSubscription = useDeepSubscription;
        this.useDeepQuery = useDeepQuery;
        this.useMinilinksQuery = (query) => useMinilinksQuery(this.minilinks, query);
        this.useMinilinksSubscription = (query) => useMinilinksSubscription(this.minilinks, query);
        this.useDeep = useDeep;
        this.DeepProvider = DeepProvider;
        this.DeepContext = DeepContext;
        this.serializeQuery = serializeQuery;
        this.serializeWhere = serializeWhere;
        this.deep = options.deep;
        if (!this.apolloClient)
            this.apolloClient = options.apolloClient;
        if (!this.deep && !options.apolloClient)
            throw new Error('options.apolloClient or options.deep is required');
        if (this.deep && !this.apolloClient && !options.apolloClient && options.token) {
            this.apolloClient = generateApolloClient({
                path: (_a = this.deep.apolloClient) === null || _a === void 0 ? void 0 : _a.path,
                ssl: (_b = this.deep.apolloClient) === null || _b === void 0 ? void 0 : _b.ssl,
                token: options.token,
            });
        }
        if (!this.apolloClient)
            throw new Error('apolloClient is invalid');
        this.client = this.apolloClient;
        this.minilinks = options.minilinks || new MinilinkCollection();
        this.table = options.table || 'links';
        this.linkId = options.linkId;
        this.token = options.token;
        this.handleAuth = (options === null || options === void 0 ? void 0 : options.handleAuth) || ((_c = options === null || options === void 0 ? void 0 : options.deep) === null || _c === void 0 ? void 0 : _c.handleAuth);
        this.linksSelectReturning = options.linksSelectReturning || options.selectReturning || 'id type_id from_id to_id value';
        this.selectReturning = options.selectReturning || this.linksSelectReturning;
        this.valuesSelectReturning = options.valuesSelectReturning || 'id link_id value';
        this.selectorsSelectReturning = options.selectorsSelectReturning || 'item_id selector_id';
        this.filesSelectReturning = options.filesSelectReturning || 'id link_id name mimeType';
        this.insertReturning = options.insertReturning || 'id';
        this.updateReturning = options.updateReturning || 'id';
        this.deleteReturning = options.deleteReturning || 'id';
        this.defaultSelectName = options.defaultSelectName || 'SELECT';
        this.defaultInsertName = options.defaultInsertName || 'INSERT';
        this.defaultUpdateName = options.defaultUpdateName || 'UPDATE';
        this.defaultDeleteName = options.defaultDeleteName || 'DELETE';
        this.silent = options.silent || false;
    }
    stringify(any) {
        if (typeof (any) === 'string') {
            let json;
            try {
                json = JSON.parse(any);
            }
            catch (e) { }
            return json ? JSON.stringify(json, null, 2) : any.toString();
        }
        else if (typeof (any) === 'object') {
            return JSON.stringify(any, null, 2);
        }
        return any;
    }
    select(exp, options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            if (!exp) {
                return { error: { message: '!exp' }, data: undefined, loading: false, networkStatus: undefined };
            }
            const query = serializeQuery(exp, (options === null || options === void 0 ? void 0 : options.table) || 'links');
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (_a = options === null || options === void 0 ? void 0 : options.returning) !== null && _a !== void 0 ? _a : (table === 'links' ? this.linksSelectReturning :
                ['strings', 'numbers', 'objects'].includes(table) ? this.valuesSelectReturning :
                    table === 'selectors' ? this.selectorsSelectReturning :
                        table === 'files' ? this.filesSelectReturning : `id`);
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultSelectName;
            const q = yield this.apolloClient.query(generateQuery({
                queries: [
                    generateQueryData({
                        tableName: table,
                        returning,
                        variables: Object.assign(Object.assign({}, variables), query)
                    }),
                ],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_c = (_b = (q)) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.q0 });
        });
    }
    ;
    insert(objects, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        return __awaiter(this, void 0, void 0, function* () {
            const _objects = Object.prototype.toString.call(objects) === '[object Array]' ? objects : [objects];
            checkAndFillShorts(_objects);
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.insertReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultInsertName;
            let q = {};
            try {
                q = yield this.apolloClient.mutate(generateSerial({
                    actions: [insertMutation(table, Object.assign(Object.assign({}, variables), { objects: _objects }), { tableName: table, operation: 'insert', returning })],
                    name: name,
                }));
            }
            catch (e) {
                const sqlError = (_d = (_c = (_b = (_a = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.extensions) === null || _c === void 0 ? void 0 : _c.internal) === null || _d === void 0 ? void 0 : _d.error;
                if (sqlError === null || sqlError === void 0 ? void 0 : sqlError.message)
                    e.message = sqlError.message;
                if (!this._silent(options))
                    throw e;
                return Object.assign(Object.assign({}, q), { data: (_g = (_f = (_e = (q)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.m0) === null || _g === void 0 ? void 0 : _g.returning, error: e });
            }
            return Object.assign(Object.assign({}, q), { data: (_k = (_j = (_h = (q)) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.m0) === null || _k === void 0 ? void 0 : _k.returning });
        });
    }
    ;
    update(exp, value, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        return __awaiter(this, void 0, void 0, function* () {
            const query = serializeQuery(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'links' : 'value');
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.updateReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultUpdateName;
            let q;
            try {
                q = yield this.apolloClient.mutate(generateSerial({
                    actions: [updateMutation(table, Object.assign(Object.assign(Object.assign({}, variables), query), { _set: value }), { tableName: table, operation: 'update', returning })],
                    name: name,
                }));
            }
            catch (e) {
                const sqlError = (_d = (_c = (_b = (_a = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.extensions) === null || _c === void 0 ? void 0 : _c.internal) === null || _d === void 0 ? void 0 : _d.error;
                if (sqlError === null || sqlError === void 0 ? void 0 : sqlError.message)
                    e.message = sqlError.message;
                if (!this._silent(options))
                    throw e;
                return Object.assign(Object.assign({}, q), { data: (_g = (_f = (_e = (q)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.m0) === null || _g === void 0 ? void 0 : _g.returning, error: e });
            }
            return Object.assign(Object.assign({}, q), { data: (_k = (_j = (_h = (q)) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.m0) === null || _k === void 0 ? void 0 : _k.returning });
        });
    }
    ;
    delete(exp, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        return __awaiter(this, void 0, void 0, function* () {
            if (!exp)
                throw new Error('!exp');
            const query = serializeQuery(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'links' : 'value');
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.deleteReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultDeleteName;
            let q;
            try {
                q = yield this.apolloClient.mutate(generateSerial({
                    actions: [deleteMutation(table, Object.assign(Object.assign(Object.assign({}, variables), query), { returning }), { tableName: table, operation: 'delete', returning })],
                    name: name,
                }));
            }
            catch (e) {
                const sqlError = (_d = (_c = (_b = (_a = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.extensions) === null || _c === void 0 ? void 0 : _c.internal) === null || _d === void 0 ? void 0 : _d.error;
                if (sqlError === null || sqlError === void 0 ? void 0 : sqlError.message)
                    e.message = sqlError.message;
                if (!this._silent(options))
                    throw e;
                return Object.assign(Object.assign({}, q), { data: (_g = (_f = (_e = (q)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.m0) === null || _g === void 0 ? void 0 : _g.returning, error: e });
            }
            return Object.assign(Object.assign({}, q), { data: (_k = (_j = (_h = (q)) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.m0) === null || _k === void 0 ? void 0 : _k.returning });
        });
    }
    ;
    serial({ name, operations, returning, silent }) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        return __awaiter(this, void 0, void 0, function* () {
            let operationsGroupedByTypeAndTable = {};
            operationsGroupedByTypeAndTable = operations.reduce((acc, operation) => {
                if (!acc[operation.type]) {
                    acc[operation.type] = {};
                }
                if (!acc[operation.type][operation.table]) {
                    acc[operation.type][operation.table] = [];
                }
                acc[operation.type][operation.table].push(operation);
                return acc;
            }, operationsGroupedByTypeAndTable);
            let serialActions = [];
            Object.keys(operationsGroupedByTypeAndTable).map((operationType) => {
                const operationsGroupedByTable = operationsGroupedByTypeAndTable[operationType];
                Object.keys(operationsGroupedByTable).map((table) => {
                    const operations = operationsGroupedByTable[table];
                    if (operationType === 'insert') {
                        const insertOperations = operations;
                        const serialAction = insertMutation(table, { objects: insertOperations.map(operation => Array.isArray(operation.objects) ? operation.objects : [operation.objects]).flat() }, { tableName: table, operation: operationType, returning });
                        serialActions.push(serialAction);
                    }
                    else if (operationType === 'update') {
                        const updateOperations = operations;
                        const newSerialActions = updateOperations.map(operation => {
                            const exp = operation.exp;
                            const value = operation.value;
                            const query = serializeQuery(exp, table === this.table || !table ? 'links' : 'value');
                            return updateMutation(table, Object.assign(Object.assign({}, query), { _set: value }), { tableName: table, operation: operationType, returning });
                        });
                        serialActions = [...serialActions, ...newSerialActions];
                    }
                    else if (operationType === 'delete') {
                        const deleteOperations = operations;
                        ;
                        const newSerialActions = deleteOperations.map(operation => {
                            const exp = operation.exp;
                            const query = serializeQuery(exp, table === this.table || !table ? 'links' : 'value');
                            return deleteMutation(table, Object.assign({}, query), { tableName: table, operation: operationType, returning });
                        });
                        serialActions = [...serialActions, ...newSerialActions];
                    }
                });
            });
            let result;
            try {
                result = yield this.apolloClient.mutate(generateSerial({
                    actions: serialActions,
                    name: name !== null && name !== void 0 ? name : 'Name',
                }));
            }
            catch (e) {
                const sqlError = (_d = (_c = (_b = (_a = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.extensions) === null || _c === void 0 ? void 0 : _c.internal) === null || _d === void 0 ? void 0 : _d.error;
                if (sqlError === null || sqlError === void 0 ? void 0 : sqlError.message)
                    e.message = sqlError.message;
                if (!silent)
                    throw e;
                return Object.assign(Object.assign({}, result), { data: (_g = (_f = (_e = (result)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.m0) === null || _g === void 0 ? void 0 : _g.returning, error: e });
            }
            return Object.assign(Object.assign({}, result), { data: ((_h = (result)) === null || _h === void 0 ? void 0 : _h.data) && Object.values((_j = (result)) === null || _j === void 0 ? void 0 : _j.data).flatMap(m => m.returning) });
        });
    }
    ;
    reserve(count) {
        return reserve({ count, client: this.apolloClient });
    }
    ;
    await(id, options = { results: false }) {
        return __awaiter(this, void 0, void 0, function* () {
            return awaitPromise({
                id, client: this.apolloClient,
                Then: yield this.id('@deep-foundation/core', 'Then'),
                Promise: yield this.id('@deep-foundation/core', 'Promise'),
                Resolved: yield this.id('@deep-foundation/core', 'Resolved'),
                Rejected: yield this.id('@deep-foundation/core', 'Rejected'),
                Results: options.results
            });
        });
    }
    ;
    id(start, ...path) {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof (start) === 'object') {
                return (_c = (_b = (_a = ((yield this.select(start)))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id;
            }
            if ((_d = _ids === null || _ids === void 0 ? void 0 : _ids[start]) === null || _d === void 0 ? void 0 : _d[path[0]]) {
                return _ids[start][path[0]];
            }
            const q = yield this.select(pathToWhere(start, ...path));
            if (q.error) {
                throw q.error;
            }
            const result = (((_f = (_e = q === null || q === void 0 ? void 0 : q.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.id) | ((_g = _ids === null || _ids === void 0 ? void 0 : _ids[start]) === null || _g === void 0 ? void 0 : _g[path === null || path === void 0 ? void 0 : path[0]]) | 0);
            if (!result && path[path.length - 1] !== true) {
                throw new Error(`Id not found by [${JSON.stringify([start, ...path])}]`);
            }
            return result;
        });
    }
    ;
    idLocal(start, ...path) {
        var _a;
        if ((_a = _ids === null || _ids === void 0 ? void 0 : _ids[start]) === null || _a === void 0 ? void 0 : _a[path[0]]) {
            return _ids[start][path[0]];
        }
        const containTypeLinkId = _ids['@deep-foundation/core'].Contain;
        const result = this.minilinks.query({
            in: Object.assign(Object.assign({ type_id: containTypeLinkId, from: Object.assign(Object.assign({}, (typeof start === 'number' && { id: start })), (typeof start === 'string' && {
                    string: {
                        value: {
                            _eq: start
                        }
                    }
                })) }, (typeof path[0] === 'string' && {
                string: {
                    value: {
                        _eq: path[0]
                    }
                }
            })), (typeof path[0] === 'boolean' && {}))
        });
        if (result.length > 0) {
            return result[0].id;
        }
        throw new Error(`Id not found by [${JSON.stringify([start, ...path])}]`);
    }
    ;
    guest(options = {}) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const relogin = typeof (options.relogin) === 'boolean' ? options.relogin : true;
            const result = yield this.apolloClient.query({ query: GUEST });
            const { linkId, token, error } = ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.guest) || {};
            if (!error && !!token && relogin) {
                if (this === null || this === void 0 ? void 0 : this.handleAuth)
                    setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(+linkId, token), 0);
            }
            return { linkId, token, error: !error && (!linkId || !token) ? 'unexepted' : error };
        });
    }
    ;
    jwt(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const relogin = typeof (options.relogin) === 'boolean' ? options.relogin : false;
            if (options === null || options === void 0 ? void 0 : options.token) {
                try {
                    const token = options === null || options === void 0 ? void 0 : options.token;
                    const decoded = parseJwt(token);
                    const linkId = decoded === null || decoded === void 0 ? void 0 : decoded.userId;
                    if (!!token && relogin) {
                        if (this === null || this === void 0 ? void 0 : this.handleAuth)
                            setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(+linkId, token), 0);
                    }
                    return { linkId, token, error: (!linkId || !token) ? 'unexepted' : undefined };
                }
                catch (e) {
                    return { error: e };
                }
            }
            else if (options === null || options === void 0 ? void 0 : options.linkId) {
                const result = yield this.apolloClient.query({ query: JWT, variables: { linkId: +options.linkId } });
                const { linkId, token, error } = ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.jwt) || {};
                if (!error && !!token && relogin) {
                    if (this === null || this === void 0 ? void 0 : this.handleAuth)
                        setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(+linkId, token), 0);
                }
                return { linkId, token, error: error ? error : (!linkId) ? 'unexepted' : undefined };
            }
            else
                return { error: `linkId or token must be provided` };
        });
    }
    ;
    whoami() {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.apolloClient.query({ query: WHOISME });
            this.linkId = (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.jwt) === null || _b === void 0 ? void 0 : _b.linkId;
            return (_d = (_c = result === null || result === void 0 ? void 0 : result.data) === null || _c === void 0 ? void 0 : _c.jwt) === null || _d === void 0 ? void 0 : _d.linkId;
        });
    }
    login(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jwt(Object.assign(Object.assign({}, options), { relogin: true }));
        });
    }
    ;
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this === null || this === void 0 ? void 0 : this.handleAuth)
                setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(0, ''), 0);
            return { linkId: 0, token: '' };
        });
    }
    ;
    can(objectIds, subjectIds, actionIds, userIds = this.linkId) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const where = {};
            if (objectIds)
                where.object_id = typeof (objectIds) === 'number' ? { _eq: +objectIds } : { _in: objectIds };
            if (subjectIds)
                where.subject_id = typeof (subjectIds) === 'number' ? { _eq: +subjectIds } : { _in: subjectIds };
            if (actionIds)
                where.action_id = typeof (actionIds) === 'number' ? { _eq: +actionIds } : { _in: actionIds };
            const result = yield this.select(where, { table: 'can', returning: 'rule_id' });
            return !!((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.length);
        });
    }
    nameLocal(input) {
        var _a, _b, _c;
        const id = typeof (input) === 'number' ? input : input.id;
        return (_c = (_b = (_a = this.minilinks.byType[this.idLocal('@deep-foundation/core', 'Contain')]) === null || _a === void 0 ? void 0 : _a.find((c) => (c === null || c === void 0 ? void 0 : c.to_id) === id)) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.value;
    }
}
export const JWT = gql `query JWT($linkId: Int) {
  jwt(input: {linkId: $linkId}) {
    linkId
    token
  }
}`;
export const WHOISME = gql `query WHOISME {
  jwt(input: {}) {
    linkId
  }
}`;
export const GUEST = gql `query GUEST {
  guest {
    linkId
    token
  }
}`;
export function useAuthNode() {
    return useLocalStore('use_auth_link_id', 0);
}
export const DeepContext = createContext(undefined);
export function useDeepGenerator(apolloClientProps) {
    const apolloClientHook = useApolloClient();
    const apolloClient = apolloClientProps || apolloClientHook;
    const [linkId, setLinkId] = useAuthNode();
    const [token, setToken] = useTokenController();
    const deep = useMemo(() => {
        if (!(apolloClient === null || apolloClient === void 0 ? void 0 : apolloClient.jwt_token)) {
            log({ token, apolloClient });
        }
        return new DeepClient({
            apolloClient, linkId, token,
            handleAuth: (linkId, token) => {
                setToken(token);
                setLinkId(linkId);
            },
        });
    }, [apolloClient]);
    return deep;
}
export function DeepProvider({ apolloClient: apolloClientProps, children, }) {
    const deep = useDeepGenerator(apolloClientProps);
    return React.createElement(DeepContext.Provider, { value: deep }, children);
}
export function useDeep() {
    return useContext(DeepContext);
}
export function useDeepQuery(query, options) {
    var _a, _b, _c;
    const [miniName] = useState((options === null || options === void 0 ? void 0 : options.mini) || Math.random().toString(36).slice(2, 7));
    debug('useDeepQuery', miniName, query, options);
    const deep = useDeep();
    const wq = useMemo(() => {
        const sq = serializeQuery(query);
        return generateQuery({
            operation: 'query',
            queries: [generateQueryData(Object.assign(Object.assign({ tableName: 'links', returning: `
          id type_id from_id to_id value
          string { id value }
          number { id value }
          object { id value }
        ` }, options), { variables: Object.assign(Object.assign({}, sq), options === null || options === void 0 ? void 0 : options.variables) }))],
            name: (options === null || options === void 0 ? void 0 : options.name) || 'USE_DEEP_QUERY',
        });
    }, [query, options]);
    const result = useQuery(wq.query, { variables: wq === null || wq === void 0 ? void 0 : wq.variables });
    useMinilinksApply(deep.minilinks, miniName, ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.q0) || []);
    const mlResult = deep.useMinilinksSubscription({ id: { _in: (_c = (_b = result === null || result === void 0 ? void 0 : result.data) === null || _b === void 0 ? void 0 : _b.q0) === null || _c === void 0 ? void 0 : _c.map(l => l.id) } });
    return Object.assign(Object.assign({}, result), { data: mlResult });
}
export function useDeepSubscription(query, options) {
    var _a, _b, _c;
    const [miniName] = useState((options === null || options === void 0 ? void 0 : options.mini) || Math.random().toString(36).slice(2, 7));
    debug('useDeepSubscription', miniName, query, options);
    const deep = useDeep();
    const wq = useMemo(() => {
        const sq = serializeQuery(query);
        return generateQuery({
            operation: 'subscription',
            queries: [generateQueryData(Object.assign(Object.assign({ tableName: 'links', returning: `
          id type_id from_id to_id value
          string { id value }
          number { id value }
          object { id value }
        ` }, options), { variables: Object.assign(Object.assign({}, sq), options === null || options === void 0 ? void 0 : options.variables) }))],
            name: (options === null || options === void 0 ? void 0 : options.name) || 'USE_DEEP_SUBSCRIPTION',
        });
    }, [query, options]);
    const result = useSubscription(wq.query, { variables: wq === null || wq === void 0 ? void 0 : wq.variables });
    useMinilinksApply(deep.minilinks, miniName, ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.q0) || []);
    const mlResult = useMinilinksSubscription(deep.minilinks, { id: { _in: (_c = (_b = result === null || result === void 0 ? void 0 : result.data) === null || _b === void 0 ? void 0 : _b.q0) === null || _c === void 0 ? void 0 : _c.map(l => l.id) } });
    return Object.assign(Object.assign({}, result), { data: mlResult });
}
export function useDeepId(start, ...path) {
    var _a, _b;
    const result = useDeepQuery({ id: { _id: [start, ...path] } });
    return { data: (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id, loading: result === null || result === void 0 ? void 0 : result.loading, error: result === null || result === void 0 ? void 0 : result.error };
}
//# sourceMappingURL=client.js.map