import _remove from 'lodash/remove.js';
import _isEqual from 'lodash/isEqual.js';
import EventEmitter from 'events';
import { useEffect, useMemo, useRef, useState } from 'react';
import Debug from 'debug';
import { minilinksQuery, minilinksQueryIs } from './minilinks-query.js';
const debug = Debug('deeplinks:minilinks');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = Debug.disable();
Debug.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
export class MinilinksLink {
    constructor(link) {
        this._applies = [];
        Object.assign(this, link);
        if (link.value) {
            if (typeof (link.value.value) === 'string' && !this.string)
                this.string = link.value;
            if (typeof (link.value.value) === 'number' && !this.number)
                this.number = link.value;
            if (typeof (link.value.value) === 'object' && !this.object)
                this.object = link.value;
        }
    }
    toPlain() {
        return {
            id: this.id,
            type_id: this.type_id,
            from_id: this.from_id,
            to_id: this.to_id,
            value: this.value,
        };
    }
    is(query) {
        return minilinksQueryIs(query, this);
    }
}
export const MinilinksGeneratorOptionsDefault = {
    id: 'id',
    type_id: 'type_id',
    type: 'type',
    typed: 'typed',
    from_id: 'from_id',
    from: 'from',
    out: 'out',
    to_id: 'to_id',
    to: 'to',
    in: 'in',
    inByType: 'inByType',
    outByType: 'outByType',
    equal: (ol, nl) => {
        return ol.type_id == nl.type_id && ol.from_id == nl.from_id && ol.to_id == nl.to_id && _isEqual(ol.value, nl.value);
    },
    Link: MinilinksLink,
};
export function Minilinks(options) {
    return function minilinks(linksArray = [], memory = {}) {
        const mc = new MinilinkCollection(options, memory);
        mc.add(linksArray);
        return mc;
    };
}
export class MinilinkCollection {
    query(query) {
        return minilinksQuery(query, this);
    }
    add(linksArray) {
        var _a, _b, _c, _d, _e, _f, _g;
        log('add', linksArray, this);
        const { byId, byFrom, byTo, byType, links, options } = this;
        const anomalies = [];
        const errors = [];
        const newLinks = [];
        for (let l = 0; l < linksArray.length; l++) {
            if (!!byId[linksArray[l][options.id]])
                errors.push(new Error(`${linksArray[l][options.id]} can't add because already exists in collection`));
            if (byId[linksArray[l][options.id]]) {
                if (options.handler)
                    options.handler(byId[linksArray[l][options.id]], this);
            }
            else {
                const link = new this.options.Link(Object.assign(Object.assign({ ml: this, _applies: [] }, linksArray[l]), { [options.typed]: [], [options.in]: [], [options.out]: [], [options.inByType]: {}, [options.outByType]: {} }));
                byId[link[options.id]] = link;
                if (link[options.from_id]) {
                    if (byFrom[link[options.from_id]])
                        byFrom[link[options.from_id]].push(link);
                    else
                        byFrom[link[options.from_id]] = [link];
                }
                if (link[options.to_id]) {
                    if (byTo[link[options.to_id]])
                        byTo[link[options.to_id]].push(link);
                    else
                        byTo[link[options.to_id]] = [link];
                }
                if (link[options.type_id]) {
                    if (byType[link[options.type_id]])
                        byType[link[options.type_id]].push(link);
                    else
                        byType[link[options.type_id]] = [link];
                }
                if ((_a = byType[link[options.id]]) === null || _a === void 0 ? void 0 : _a.length) {
                    for (let i = 0; i < ((_b = byType[link[options.id]]) === null || _b === void 0 ? void 0 : _b.length); i++) {
                        const dep = byType[link[options.id]][i];
                        dep.type = link;
                        link[options.typed].push(dep);
                    }
                }
                if ((_c = byFrom[link[options.id]]) === null || _c === void 0 ? void 0 : _c.length) {
                    for (let i = 0; i < ((_d = byFrom[link[options.id]]) === null || _d === void 0 ? void 0 : _d.length); i++) {
                        const dep = byFrom[link[options.id]][i];
                        dep[options.from] = link;
                        link[options.out].push(dep);
                        link[options.outByType][dep[options.type_id]] = link[options.outByType][dep[options.type_id]] || [];
                        link[options.outByType][dep[options.type_id]].push(dep);
                    }
                }
                if ((_e = byTo[link[options.id]]) === null || _e === void 0 ? void 0 : _e.length) {
                    for (let i = 0; i < ((_f = byTo[link[options.id]]) === null || _f === void 0 ? void 0 : _f.length); i++) {
                        const dep = byTo[link[options.id]][i];
                        dep[options.to] = link;
                        link[options.in].push(dep);
                        link[options.inByType][dep[options.type_id]] = link[options.inByType][dep[options.type_id]] || [];
                        link[options.inByType][dep[options.type_id]].push(dep);
                    }
                }
                links.push(link);
                newLinks.push(link);
            }
        }
        for (let l = 0; l < newLinks.length; l++) {
            const link = newLinks[l];
            const type = byId[link[options.type_id]];
            const from = byId[link[options.from_id]];
            const to = byId[link[options.to_id]];
            if (type) {
                link[options.type] = type;
                if (!type[options.typed].find(l => l.id === link.id))
                    type[options.typed].push(link);
            }
            else if (link[options.type_id])
                anomalies.push(new Error(`${link[options.id]} link.type_id ${link[options.type_id]} not founded`));
            if (from) {
                link[options.from] = from;
                if (!from[options.out].find(l => l.id === link.id))
                    from[options.out].push(link);
                from[options.outByType][link[options.type_id]] = from[options.outByType][link[options.type_id]] || [];
                if (!((_g = from === null || from === void 0 ? void 0 : from[options.outByType]) === null || _g === void 0 ? void 0 : _g[link[options.type_id]].find(l => l.id === link.id)))
                    from[options.outByType][link[options.type_id]].push(link);
            }
            else if (link[options.from_id])
                anomalies.push(new Error(`${link[options.id]} link.from_id ${link[options.from_id]} not founded`));
            if (to) {
                link[options.to] = to;
                if (!to[options.in].find(l => l.id === link.id))
                    to[options.in].push(link);
                to[options.inByType][link[options.type_id]] = to[options.inByType][link[options.type_id]] || [];
                if (!to[options.inByType][link[options.type_id]].find(l => l.id === link.id))
                    to[options.inByType][link[options.type_id]].push(link);
            }
            else if (link[options.to_id])
                anomalies.push(new Error(`${link[options.id]} link.to_id ${link[options.to_id]} not founded`));
            if (options.handler)
                options.handler(link, this);
        }
        for (let l = 0; l < newLinks.length; l++) {
            const link = newLinks[l];
            if (!this._updating)
                this.emitter.emit('added', undefined, link);
        }
        return {
            anomalies,
            errors,
        };
    }
    remove(idsArray) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        log('remove', idsArray, this);
        const { byId, byFrom, byTo, byType, types, links, options } = this;
        const anomalies = [];
        const errors = [];
        const oldLinksArray = [];
        const oldLinksObject = {};
        for (let l = 0; l < idsArray.length; l++) {
            const id = idsArray[l];
            const link = byId[id];
            log('remove old l:', l, 'id:', id, 'link:', link);
            if (link) {
                oldLinksArray.push(link);
                oldLinksObject[id] = link;
            }
        }
        for (let l = 0; l < idsArray.length; l++) {
            const id = idsArray[l];
            const link = oldLinksArray[l];
            if (!link)
                errors.push(new Error(`${id} can't delete because not exists in collection`));
            _remove((_a = link === null || link === void 0 ? void 0 : link[options.to]) === null || _a === void 0 ? void 0 : _a[options.in], (r) => r.id === id);
            _remove((_b = link === null || link === void 0 ? void 0 : link[options.from]) === null || _b === void 0 ? void 0 : _b[options.out], (r) => r.id === id);
            _remove((byFrom === null || byFrom === void 0 ? void 0 : byFrom[link === null || link === void 0 ? void 0 : link[options.from_id]]) || [], (r) => r.id === id);
            _remove((byTo === null || byTo === void 0 ? void 0 : byTo[link === null || link === void 0 ? void 0 : link[options.to_id]]) || [], (r) => r.id === id);
            _remove((byType === null || byType === void 0 ? void 0 : byType[link === null || link === void 0 ? void 0 : link[options.type_id]]) || [], (r) => r.id === id);
            _remove(((_d = (_c = link === null || link === void 0 ? void 0 : link[options.from]) === null || _c === void 0 ? void 0 : _c.outByType) === null || _d === void 0 ? void 0 : _d[link.type_id]) || [], (r) => r.id === id);
            _remove(((_f = (_e = link === null || link === void 0 ? void 0 : link[options.to]) === null || _e === void 0 ? void 0 : _e.inByType) === null || _f === void 0 ? void 0 : _f[link.type_id]) || [], (r) => r.id === id);
            for (let i = 0; i < ((_g = byFrom === null || byFrom === void 0 ? void 0 : byFrom[id]) === null || _g === void 0 ? void 0 : _g.length); i++) {
                const dep = (_h = byFrom === null || byFrom === void 0 ? void 0 : byFrom[id]) === null || _h === void 0 ? void 0 : _h[i];
                dep[options.from] = undefined;
            }
            for (let i = 0; i < ((_j = byTo === null || byTo === void 0 ? void 0 : byTo[id]) === null || _j === void 0 ? void 0 : _j.length); i++) {
                const dep = (_k = byTo === null || byTo === void 0 ? void 0 : byTo[id]) === null || _k === void 0 ? void 0 : _k[i];
                dep[options.to] = undefined;
            }
            for (let i = 0; i < ((_l = byType === null || byType === void 0 ? void 0 : byType[id]) === null || _l === void 0 ? void 0 : _l.length); i++) {
                const dep = (_m = byType === null || byType === void 0 ? void 0 : byType[id]) === null || _m === void 0 ? void 0 : _m[i];
                dep[options.type] = undefined;
            }
            byId === null || byId === void 0 ? true : delete byId[id];
        }
        _remove(links, l => idsArray.includes(l[options.id]));
        for (let l = 0; l < oldLinksArray.length; l++) {
            const link = oldLinksArray[l];
            log('emit removed link', link, '_updating', this._updating);
            if (!this._updating)
                this.emitter.emit('removed', link);
        }
        return {
            anomalies,
            errors,
        };
    }
    apply(linksArray, applyName = '') {
        log('apply', linksArray, this);
        const { byId, byFrom, byTo, byType, types, links, options } = this;
        const toAdd = [];
        const toUpdate = [];
        const beforeUpdate = {};
        const toRemove = [];
        const _byId = {};
        for (let l = 0; l < linksArray.length; l++) {
            const link = linksArray[l];
            const old = byId[link.id];
            if (!old) {
                link._applies = [applyName];
                this.emitter.emit('apply', old, link);
                toAdd.push(link);
            }
            else {
                const index = old._applies.indexOf(applyName);
                if (!~index) {
                    link._applies = old._applies = [...old._applies, applyName];
                    this.emitter.emit('apply', old, link);
                }
                else {
                    link._applies = old._applies;
                }
                if (!options.equal(old, link)) {
                    toUpdate.push(link);
                    beforeUpdate[link.id] = old;
                }
            }
            _byId[link.id] = link;
        }
        for (let l = 0; l < links.length; l++) {
            const link = links[l];
            if (!_byId[link.id]) {
                const index = link._applies.indexOf(applyName);
                if (!!~index) {
                    if (link._applies.length === 1) {
                        toRemove.push(link);
                    }
                    else {
                        link._applies.splice(index, 1);
                        this.emitter.emit('apply', link, link);
                    }
                }
            }
        }
        const r1 = this.remove(toRemove.map(l => l[options.id]));
        const a1 = this.add(toAdd);
        this._updating = true;
        const r2 = this.remove(toUpdate.map(l => l[options.id]));
        const a2 = this.add(toUpdate);
        for (let i = 0; i < toUpdate.length; i++) {
            const l = toUpdate[i];
            this.emitter.emit('updated', beforeUpdate[l.id], byId[l.id]);
        }
        this._updating = false;
        return { errors: [...r1.errors, ...a1.errors, ...r2.errors, ...a2.errors], anomalies: [...r1.anomalies, ...a1.anomalies, ...r2.anomalies, ...a2.anomalies] };
    }
    constructor(options, memory) {
        this.useMinilinksQuery = useMinilinksQuery;
        this.useMinilinksFilter = useMinilinksFilter;
        this.useMinilinksApply = useMinilinksApply;
        this.useMinilinksSubscription = useMinilinksSubscription;
        this.useMinilinksHandle = useMinilinksHandle;
        this.types = {};
        this.byId = {};
        this.byFrom = {};
        this.byTo = {};
        this.byType = {};
        this.links = [];
        this._updating = false;
        const _options = options || MinilinksGeneratorOptionsDefault;
        this.types = this.byType = (memory === null || memory === void 0 ? void 0 : memory.types) || {};
        this.byId = (memory === null || memory === void 0 ? void 0 : memory.byId) || {};
        this.byFrom = (memory === null || memory === void 0 ? void 0 : memory.byFrom) || {};
        this.byTo = (memory === null || memory === void 0 ? void 0 : memory.byTo) || {};
        this.links = (memory === null || memory === void 0 ? void 0 : memory.links) || [];
        this.options = _options;
        this.emitter = new EventEmitter();
    }
}
export const minilinks = Minilinks(MinilinksGeneratorOptionsDefault);
export function useMinilinksConstruct(options) {
    const mlRef = useRef(useMemo(() => {
        return new MinilinkCollection(options);
    }, []));
    const ml = mlRef.current;
    return { ml, ref: mlRef };
}
export function useMinilinksFilter(ml, filter, results, interval) {
    const [state, setState] = useState();
    useEffect(() => {
        const addedListener = (ol, nl) => {
            if (filter(nl, ol, nl)) {
                setState(results(nl, ml, ol, nl));
            }
        };
        ml.emitter.on('added', addedListener);
        const updatedListener = (ol, nl) => {
            if (filter(nl, ol, nl)) {
                setState(results(nl, ml, ol, nl));
            }
        };
        ml.emitter.on('updated', updatedListener);
        const removedListener = (ol, nl) => {
            if (filter(ol, ol, nl)) {
                setState(results(ol, ml, ol, nl));
            }
        };
        ml.emitter.on('removed', removedListener);
        const applyListener = (ol, nl) => {
            if (filter(nl, ol, nl)) {
                setState(results(nl, ml, ol, nl));
            }
        };
        let timeout;
        if (interval)
            timeout = setTimeout(() => {
                results(undefined, ml);
            }, interval);
        ml.emitter.on('apply', applyListener);
        return () => {
            clearTimeout(timeout);
            ml.emitter.removeListener('added', addedListener);
            ml.emitter.removeListener('updated', updatedListener);
            ml.emitter.removeListener('removed', removedListener);
            ml.emitter.removeListener('apply', applyListener);
        };
    }, []);
    useEffect(() => {
        setState(results(undefined, ml, undefined, undefined));
    }, [filter, results]);
    return state;
}
;
export function useMinilinksHandle(ml, handler) {
    useEffect(() => {
        const addedListener = (ol, nl) => {
            handler('added', ol, nl);
        };
        ml.emitter.on('added', addedListener);
        const updatedListener = (ol, nl) => {
            handler('updated', ol, nl);
        };
        ml.emitter.on('updated', updatedListener);
        const removedListener = (ol, nl) => {
            handler('removed', ol, nl);
        };
        ml.emitter.on('removed', removedListener);
        const applyListener = (ol, nl) => {
            handler('apply', ol, nl);
        };
        ml.emitter.on('apply', applyListener);
        return () => {
            ml.emitter.removeListener('added', addedListener);
            ml.emitter.removeListener('updated', updatedListener);
            ml.emitter.removeListener('removed', removedListener);
            ml.emitter.removeListener('apply', applyListener);
        };
    }, []);
}
;
export function useMinilinksApply(ml, name, data) {
    const [strictName] = useState(name);
    useEffect(() => {
        return () => {
            ml.apply([], strictName);
        };
    }, []);
    ml.apply(data, strictName);
}
export function useMinilinksQuery(ml, query) {
    return useMemo(() => ml.query(query), [ml, query]);
}
;
export function useMinilinksSubscription(ml, query) {
    const [iteration, setIteration] = useState(0);
    useEffect(() => {
        const iterationsInterval = setInterval(() => {
            setIteration(iteration === Number.MAX_SAFE_INTEGER ? 0 : iteration + 1);
        }, 1000);
        return () => clearInterval(iterationsInterval);
    }, []);
    const data = ml.query(query);
    return data;
}
;
//# sourceMappingURL=minilinks.js.map