var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import atob from 'atob';
import { URL } from 'url';
import express from 'express';
import router from './imports/router/index.js';
import generateJwtServer from './imports/router/jwt.js';
import generateGuestServer from './imports/router/guest.js';
import generatePackagerServer from './imports/router/packager.js';
import generateAuthorizationServer from './imports/router/authorization.js';
import axios from 'axios';
import http from 'http';
import { createProxyMiddleware, responseInterceptor } from 'http-proxy-middleware';
import { createRequire } from "module";
const require = createRequire(import.meta.url);
const expressPlayground = require('graphql-playground-middleware-express').default;
import moesif from 'moesif-nodejs';
import Debug from 'debug';
import waitOn from 'wait-on';
import { generateApolloClient } from '@deep-foundation/hasura/client.js';
import { DeepClient } from './imports/client.js';
import gql from 'graphql-tag';
import { containerController, DOCKER, getJwt } from './imports/router/links.js';
import _ from 'lodash';
import Cors from 'cors';
const DEEPLINKS_HASURA_PATH = process.env.DEEPLINKS_HASURA_PATH || 'localhost:8080';
const DEEPLINKS_HASURA_STORAGE_URL = process.env.DEEPLINKS_HASURA_STORAGE_URL || 'http://localhost:8000';
const DEEPLINKS_HASURA_SSL = process.env.DEEPLINKS_HASURA_SSL || 0;
const DEEPLINKS_HASURA_SECRET = process.env.DEEPLINKS_HASURA_SECRET || 'myadminsecretkey';
const MOESIF_TOKEN = process.env.MOESIF_TOKEN || '';
const DEEPLINKS_PUBLIC_URL = process.env.DEEPLINKS_PUBLIC_URL || '';
const debug = Debug('deeplinks');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = Debug.disable();
Debug.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
export const delay = (time) => new Promise(res => setTimeout(() => res(null), time));
const app = express();
const httpServer = http.createServer(app);
app.get('/gql', expressPlayground({
    tabs: [{
            endpoint: `${DEEPLINKS_PUBLIC_URL}/gql`,
            query: `query MyQuery {
      links(limit: 1) {
        id
      }
    }`,
            headers: {
                Authorization: 'Bearer TOKEN',
            },
        }],
}));
app.use('/gql', createProxyMiddleware({
    target: `http${DEEPLINKS_HASURA_SSL === '1' ? 's' : ''}://${DEEPLINKS_HASURA_PATH}`,
    changeOrigin: true,
    ws: true,
    logLevel: 'debug',
    pathRewrite: {
        "/gql": "/v1/graphql",
    },
}));
app.get(['/file'], createProxyMiddleware({
    target: DEEPLINKS_HASURA_STORAGE_URL,
    changeOrigin: true,
    logLevel: 'debug',
    pathRewrite: (path, req) => __awaiter(void 0, void 0, void 0, function* () {
        console.log('/file get proxy', 'path', path);
        const headers = req.headers;
        console.log('/file get proxy', '{headers: headers}', { headers: headers });
        const newurl = new URL(`${headers['x-forwarded-proto']}://${headers['host']}${path}`);
        console.log('/file get proxy', 'newurl', newurl);
        const linkId = newurl.searchParams.get('linkId');
        console.log('/file get proxy', 'newurl.searchParams', newurl.searchParams);
        console.log('/file get proxy', 'linkId', linkId);
        const result = yield deep.apolloClient.query({
            query: gql `{
        files(where: {link_id: {_eq: ${linkId}}}) {
          id
        }
      }`
        });
        console.log('/file get proxy', 'result', result);
        console.log('/file get proxy', 'result.data.files[0].id', result.data.files[0].id);
        const fileId = result.data.files[0].id;
        return `/v1/files/${fileId}`;
    })
}));
const cors = Cors({ methods: ['POST', 'OPTIONS'] });
app.use(cors);
app.post('/file', (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('DEEPLINKS_HASURA_STORAGE_URL', DEEPLINKS_HASURA_STORAGE_URL);
    const headers = req.headers;
    let userId;
    let linkId;
    try {
        const claims = atob(`${headers['authorization'] ? headers['authorization'] : headers['Authorization']}`.split(' ')[1].split('.')[1]);
        userId = +(JSON.parse(claims)['https://hasura.io/jwt/claims']['x-hasura-user-id']);
        linkId = +(headers['linkId'] || headers['linkid']);
        console.log('linkId', linkId);
    }
    catch (e) {
        console.log('error: ', e);
    }
    if (!userId)
        res.status(403).send('Update CAN NOT be processes');
    const canResult = (yield deep.can(linkId, userId, yield deep.id('@deep-foundation/core', 'AllowUpdateType'))) || userId === (yield deep.id('deep', 'admin'));
    console.log('can', yield deep.can(linkId, userId, yield deep.id('@deep-foundation/core', 'AllowUpdateType')), 'isAdmin', userId === (yield deep.id('deep', 'admin')));
    console.log('userId', userId, typeof (userId));
    console.log('canResult', canResult);
    if (!canResult)
        return res.status(403).send(`You cant update link ##${linkId} as user ##${userId}, and user ##${userId} is not admin.`);
    yield createProxyMiddleware({
        target: DEEPLINKS_HASURA_STORAGE_URL,
        selfHandleResponse: true,
        logLevel: 'debug',
        onError: (err, req, res, target) => {
            console.log('onError', err);
            res.writeHead(500, {
                'Content-Type': 'text/plain',
            });
            res.end('Something went wrong. And we are reporting a custom error message.');
        },
        onProxyRes: responseInterceptor((responseBuffer, proxyRes, req, res) => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b, _c;
            console.log('onProxyRes');
            const response = responseBuffer.toString('utf8');
            console.log(`RESPONSE ${response}`);
            let files;
            try {
                files = JSON.parse(response);
                console.log('files', files);
                if (!files)
                    return response;
                const UPDATE_FILE_LINKID = gql `mutation UPDATE_FILE_LINKID($linkId: bigint, $fileid: uuid, $uploadedByLinkId: bigint) {
          updateFiles(where: {id: {_eq: $fileid}}, _set: {link_id: $linkId, uploadedByLinkId: $uploadedByLinkId }){
            returning {
              id
              link_id
              uploadedByLinkId
            }
          }
        }`;
                console.log('files[0].id', files.id);
                const updated = yield client.mutate({
                    mutation: UPDATE_FILE_LINKID,
                    variables: {
                        fileid: files.id,
                        linkId: linkId,
                        uploadedByLinkId: userId
                    },
                });
                console.log('linkid', linkId);
                console.log('data', (_b = (_a = updated === null || updated === void 0 ? void 0 : updated.data) === null || _a === void 0 ? void 0 : _a.updateFiles) === null || _b === void 0 ? void 0 : _b.returning);
            }
            catch (e) {
                console.log('try error: ', e);
                if ((_c = files[0]) === null || _c === void 0 ? void 0 : _c.id) {
                    yield client.mutate({
                        mutation: gql `mutation DELETE_FILE($fileid: uuid) { deleteFiles(where: {id: {_eq: $fileid}}){ returning { id } } }`,
                        variables: {
                            fileid: files.id,
                        },
                    });
                    return JSON.stringify({ error: 'one link - one file' });
                }
            }
            return response;
        })),
        changeOrigin: true,
        pathRewrite: {
            "/file": "/v1/files",
        },
    })(req, res, next);
}));
app.use(['/v1', '/v1alpha1', '/v2', '/console'], createProxyMiddleware({
    target: `http${DEEPLINKS_HASURA_SSL === '1' ? 's' : ''}://${DEEPLINKS_HASURA_PATH}`,
    changeOrigin: true,
    ws: true,
    logLevel: 'debug',
}));
if (MOESIF_TOKEN) {
    const moesifMiddleware = moesif({ applicationId: MOESIF_TOKEN });
    app.use(moesifMiddleware);
    moesifMiddleware.startCaptureOutgoing();
}
app.use(express.json());
app.use('/', router);
const start = () => __awaiter(void 0, void 0, void 0, function* () {
    const jwtServer = generateJwtServer(httpServer);
    const authorizationServer = generateAuthorizationServer(httpServer);
    const guestServer = generateGuestServer(httpServer);
    const packagerServer = generatePackagerServer(httpServer);
    yield jwtServer.start();
    yield guestServer.start();
    yield authorizationServer.start();
    yield packagerServer.start();
    jwtServer.applyMiddleware({ path: '/api/jwt', app });
    guestServer.applyMiddleware({ path: '/api/guest', app });
    authorizationServer.applyMiddleware({ path: '/api/authorization', app });
    packagerServer.applyMiddleware({ path: '/api/packager', app });
    yield new Promise(resolve => httpServer.listen({ port: process.env.PORT }, resolve));
    log(`Hello bugfixers! Listening ${process.env.PORT} port`);
    try {
        yield waitOn({ resources: [`http${DEEPLINKS_HASURA_SSL === '1' ? 's' : ''}-get://${DEEPLINKS_HASURA_PATH}/console`] });
        yield axios({
            method: 'post',
            url: `http${DEEPLINKS_HASURA_SSL === '1' ? 's' : ''}://${DEEPLINKS_HASURA_PATH}/v1/metadata`,
            headers: { 'x-hasura-admin-secret': DEEPLINKS_HASURA_SECRET, 'Content-Type': 'application/json' },
            data: { type: 'reload_metadata', args: {} }
        }).then(() => {
            log('hasura metadata reloaded');
        }, () => {
            error('hasura metadata broken');
        });
    }
    catch (e) {
        error(e);
    }
});
start();
const client = generateApolloClient({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new DeepClient({
    apolloClient: client,
});
const routesDebug = Debug('deeplinks').extend('eh').extend('routes');
const routesDebugLog = routesDebug.extend('log');
const routesDebugError = routesDebug.extend('error');
let currentServers = {};
let currentPorts = {};
let busy = false;
let portTypeId = 0;
const toJSON = (data) => JSON.stringify(data, Object.getOwnPropertyNames(data), 2);
const handleRoutes = () => __awaiter(void 0, void 0, void 0, function* () {
    var _d, _e, _f, _g, _h, _j, _k, _l;
    if (busy)
        return;
    busy = true;
    try {
        portTypeId = yield deep.id('@deep-foundation/core', 'Port');
        const handleRouteTypeId = yield deep.id('@deep-foundation/core', 'HandleRoute');
        const routerStringUseTypeId = yield deep.id('@deep-foundation/core', 'RouterStringUse');
        const routerListeningTypeId = yield deep.id('@deep-foundation/core', 'RouterListening');
        const routesResult = yield client.query({
            query: gql `
        query {
          ports: links(where: {
            type_id: { _eq: "${portTypeId}" }
          }) {
            id
            port: value
            routerListening: in(where: {
              type_id: { _eq: "${routerListeningTypeId}" }
            }) {
              id
              router: from {
                id
                routerStringUse: in(where: {
                  type_id: { _eq: "${routerStringUseTypeId}" }
                }) {
                  id
                  routeString: value
                  route: from {
                    id
                    handleRoute: out(where: {
                      type_id: { _eq: "${handleRouteTypeId}" }
                    }) {
                      id
                      handler: to {
                        id
                        supports: from {
                          id
                          isolation: from {
                            id
                            image: value
                          }
                        }
                        file: to {
                          id
                          code: value
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      `, variables: {}
        });
        const portsResult = routesResult.data.ports;
        routesDebugLog('portsResult', JSON.stringify(portsResult, null, 2));
        const ports = {};
        for (const port of portsResult) {
            const portValue = (_d = port === null || port === void 0 ? void 0 : port.port) === null || _d === void 0 ? void 0 : _d.value;
            ports[portValue] = port;
        }
        routesDebugLog('ports', JSON.stringify(ports, null, 2));
        const updatedOrAddedPorts = [];
        for (const key in currentPorts) {
            if (currentPorts.hasOwnProperty(key)) {
                if (ports.hasOwnProperty(key)) {
                    if (!_.isEqual(currentPorts[key], ports[key])) {
                        currentPorts[key] = ports[key];
                        updatedOrAddedPorts.push(ports[key]);
                    }
                    else {
                    }
                }
                else {
                    if (currentServers.hasOwnProperty(key)) {
                        const element = currentServers[key];
                        element.close();
                        delete currentServers[key];
                    }
                    delete currentPorts[key];
                }
            }
        }
        for (const key in ports) {
            if (ports.hasOwnProperty(key)) {
                if (!currentPorts.hasOwnProperty(key)) {
                    currentPorts[key] = ports[key];
                    updatedOrAddedPorts.push(ports[key]);
                }
            }
        }
        routesDebugLog('updatedOrAddedPorts', JSON.stringify(updatedOrAddedPorts, null, 2));
        routesDebugLog('currentPorts', JSON.stringify(currentPorts, null, 2));
        const imageContainers = {};
        updatedOrAddedPorts.forEach(port => {
            port.routerListening.forEach(routerListening => {
                var _a;
                (_a = routerListening === null || routerListening === void 0 ? void 0 : routerListening.router) === null || _a === void 0 ? void 0 : _a.routerStringUse.forEach(routerStringUse => {
                    var _a;
                    (_a = routerStringUse === null || routerStringUse === void 0 ? void 0 : routerStringUse.route) === null || _a === void 0 ? void 0 : _a.handleRoute.forEach(handleRoute => {
                        var _a, _b, _c, _d;
                        imageContainers[(_d = (_c = (_b = (_a = handleRoute === null || handleRoute === void 0 ? void 0 : handleRoute.handler) === null || _a === void 0 ? void 0 : _a.supports) === null || _b === void 0 ? void 0 : _b.isolation) === null || _c === void 0 ? void 0 : _c.image) === null || _d === void 0 ? void 0 : _d.value] = {};
                    });
                });
            });
        });
        const imageList = Object.keys(imageContainers);
        routesDebugLog('imageList', imageList);
        for (const image of imageList) {
            routesDebugLog(`preparing container ${image}`);
            imageContainers[image] = yield containerController.newContainer({
                handler: image,
                forceRestart: true,
                publish: +DOCKER ? false : true,
                code: '',
                jwt: '',
                data: {}
            });
        }
        for (const port of updatedOrAddedPorts) {
            const portValue = (_e = port === null || port === void 0 ? void 0 : port.port) === null || _e === void 0 ? void 0 : _e.value;
            if (currentServers.hasOwnProperty(portValue)) {
                currentServers[portValue].close();
            }
            if (port.routerListening.length > 0) {
                routesDebugLog(`listening on port ${portValue}`);
                const portServer = express();
                currentServers[portValue] = http.createServer({ maxHeaderSize: 10 * 1024 * 1024 * 1024 }, portServer).listen(portValue);
                for (const routerListening of port.routerListening) {
                    const router = routerListening === null || routerListening === void 0 ? void 0 : routerListening.router;
                    for (const routerStringUse of router.routerStringUse) {
                        const routeString = (_f = routerStringUse === null || routerStringUse === void 0 ? void 0 : routerStringUse.routeString) === null || _f === void 0 ? void 0 : _f.value;
                        routesDebugLog(`route string ${routeString}`);
                        const route = routerStringUse === null || routerStringUse === void 0 ? void 0 : routerStringUse.route;
                        for (const handleRoute of route.handleRoute) {
                            const handler = handleRoute === null || handleRoute === void 0 ? void 0 : handleRoute.handler;
                            const handlerId = handler === null || handler === void 0 ? void 0 : handler.id;
                            routesDebugLog(`handler id ${handlerId}`);
                            const jwt = yield getJwt(handlerId, routesDebugLog);
                            routesDebugLog(`jwt ${jwt}`);
                            const image = (_j = (_h = (_g = handler === null || handler === void 0 ? void 0 : handler.supports) === null || _g === void 0 ? void 0 : _g.isolation) === null || _h === void 0 ? void 0 : _h.image) === null || _j === void 0 ? void 0 : _j.value;
                            routesDebugLog(`image`, image);
                            const container = imageContainers[image];
                            routesDebugLog(`container`, JSON.stringify(container, null, 2));
                            const code = (_l = (_k = handler === null || handler === void 0 ? void 0 : handler.file) === null || _k === void 0 ? void 0 : _k.code) === null || _l === void 0 ? void 0 : _l.value;
                            routesDebugLog(`code ${code}`);
                            routesDebugLog('container', container);
                            const proxy = createProxyMiddleware({
                                target: `http://${container.host}:${container.port}`,
                                changeOrigin: true,
                                ws: true,
                                pathRewrite: {
                                    [routeString]: "/http-call",
                                },
                                onProxyReq: (proxyReq, req, res) => {
                                    routesDebugLog('deeplinks request');
                                    routesDebugLog('req.method', req.method);
                                    routesDebugLog('req.body', req.body);
                                    proxyReq.setHeader('deep-call-options', encodeURI(JSON.stringify({
                                        jwt,
                                        code,
                                        data: {},
                                    })));
                                },
                                onProxyRes: (proxyRes, req, res) => {
                                    proxyRes.on('data', function (data) {
                                        return __awaiter(this, void 0, void 0, function* () {
                                            try {
                                                data = data.toString('utf-8');
                                                routesDebugLog('data', data);
                                                if (data.startsWith('{')) {
                                                    data = JSON.parse(data);
                                                    if (data.hasOwnProperty('rejected')) {
                                                        routesDebugLog('rejected', data.rejected);
                                                        const handlingErrorTypeId = yield deep.id('@deep-foundation/core', 'HandlingError');
                                                        routesDebugLog('handlingErrorTypeId', handlingErrorTypeId);
                                                        const insertResult = yield deep.insert({
                                                            type_id: handlingErrorTypeId,
                                                            object: { data: { value: data.rejected } },
                                                            out: { data: [
                                                                    {
                                                                        type_id: yield deep.id('@deep-foundation/core', 'HandlingErrorReason'),
                                                                        to_id: route.id
                                                                    },
                                                                    {
                                                                        type_id: yield deep.id('@deep-foundation/core', 'HandlingErrorReason'),
                                                                        to_id: handleRoute.id
                                                                    }
                                                                ] },
                                                        }, {
                                                            name: 'INSERT_HANDLING_ERROR',
                                                        });
                                                    }
                                                }
                                            }
                                            catch (e) {
                                                routesDebugError('deeplinks response error', e);
                                            }
                                        });
                                    });
                                }
                            });
                            portServer.use(routeString, proxy);
                        }
                    }
                }
            }
        }
    }
    catch (e) {
        routesDebugLog(toJSON(e));
    }
    busy = false;
});
const startRouteHandling = () => __awaiter(void 0, void 0, void 0, function* () {
    setInterval(handleRoutes, 5000);
});
startRouteHandling();
//# sourceMappingURL=index.js.map